from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

AMPLITUDE_API_KEY = getattr(settings, 'AMPLITUDE_API_KEY')
if not AMPLITUDE_API_KEY:
    raise ImproperlyConfigured('"AMPLITUDE_API_KEY" is not set')

AMPLITUDE_INCLUDE_USER_DATA = getattr(settings, 'AMPLITUDE_INCLUDE_USER_DATA', False)  # NOQA: E501
AMPLITUDE_INCLUDE_GROUP_DATA = getattr(settings, 'AMPLITUDE_INCLUDE_GROUP_DATA', False)  # NOQA: E501



import logging
import time
from datetime import datetime, timedelta
from typing import Any, Dict, List, Union

import httpx
from django.conf import settings
from django.urls import resolve

from . import settings as app_settings
from .utils import get_client_ip, get_user_agent

try:
    from django.contrib.gis.geoip2 import GeoIP2  # type: ignore
except ImportError:
    CAN_GEOIP = False
else:  # pragma: no cover
    CAN_GEOIP = True


log = logging.getLogger(__name__)


class AmplitudeException(Exception):
    pass


class Amplitude():

    def __init__(
        self,
        api_key: str = None,
        include_user_data: bool = None,
        include_group_data: bool = None,
    ):
        if not api_key:
            api_key = app_settings.AMPLITUDE_API_KEY
        if include_user_data is None:
            include_user_data = app_settings.AMPLITUDE_INCLUDE_USER_DATA
        if include_group_data is None:
            include_group_data = app_settings.AMPLITUDE_INCLUDE_GROUP_DATA

        self.url = 'https://api.amplitude.com/2/httpapi'
        self.api_key = api_key
        self.include_user_data = include_user_data
        self.include_group_data = include_group_data

    def send_events(self, events: List[Dict[str, Any]]) -> dict:
        """
        https://developers.amplitude.com/docs/http-api-v2
        """
        events = [self.clean_event(event) for event in events]
        kwargs: Dict[str, Any] = {
            'url': self.url,
            'method': 'POST',
            'json': {
                'events': events,
                'api_key': self.api_key
            }
        }
        response = httpx.request(**kwargs)
        try:
            response.raise_for_status()
        except httpx.HTTPError as e:
            raise AmplitudeException(e)

        return response.json()

    def clean_event(self, event: dict) -> dict:
        for key, value in event.items():
            if isinstance(value, dict):
                event[key] = {k: v for k, v in value.items() if v is not [None, [], '', {}]}  # NOQA: E501

        event = {k: v for k, v in event.items() if v not in [None, [], '', {}]}

        return event

    def page_view_event(self, request):
        """
        Send an event based on a Django request
        """
        url_name = resolve(request.path_info).url_name

        event: Dict[str, Any] = {
            'event_type': f'Page view {url_name}',
            'time': int(round(time.time() * 1000)),
            'ip': get_client_ip(request),
            'language': getattr(request, 'LANGUAGE_CODE', ''),
            # 'app_version': '',
            # 'device_id': '',
            # 'carrier': '',
            # 'dma': 'San Francisco-Oakland-San Jose, CA',
            # 'price': 4.99,
            # 'quantity': 3,
            # 'revenue': -1.99,
            # 'productId': 'Google Pay Store Product Id',
            # 'revenueType': 'Refund',
            # 'idfa': '',
            # 'idfv': '',
            # 'adid': '',
            # 'android_id': '',
            # 'event_id': 23,
            # 'insert_id': '',
        }
        if request.user.is_authenticated:
            event['user_id'] = f'{request.user.id:05}'
        event['session_id'] = self.session_id_from_request(request)
        event['event_properties'] = self.event_properties_from_request(request)
        event['user_properties'] = self.user_properties_from_request(request)
        event['groups'] = self.group_from_request(request)
        device_data = self.device_data_from_request(request)
        event.update(device_data)
        location_data = self.location_data_from_ip_address(event['ip'])
        event.update(location_data)

        self.send_events(events=[event])

    def session_id_from_request(self, request) -> Union[int, None]:
        if not request.session:
            return None

        session_end = request.session.get_expiry_date()
        session_start = session_end - timedelta(seconds=settings.SESSION_COOKIE_AGE)  # NOQA: E501
        epoch = datetime.utcfromtimestamp(0)
        start_epoch = (session_start - epoch).total_seconds() * 1000.0
        return int(start_epoch)

    def event_properties_from_request(self, request) -> dict:
        url_name = resolve(request.path_info).url_name
        event_properties = {
            'url': request.path,
            'url_name': url_name,
            'method': request.method,
            'params': dict(request.GET),
        }
        if request.resolver_match:
            event_properties['kwargs'] = request.resolver_match.kwargs
        return event_properties

    def user_properties_from_request(self, request) -> dict:
        if not self.include_user_data or not request.user.is_authenticated:
            return {}

        return {
            'username': request.user.get_username(),
            'email': request.user.email,
            'full_name': request.user.get_full_name(),
            'is_staff': request.user.is_staff,
            'is_superuser': request.user.is_superuser,
            'last_login': request.user.last_login.isoformat(),
            'date_joined': request.user.date_joined.isoformat(),
        }

    def group_from_request(self, request) -> list:
        if not self.include_group_data or not request.user.is_authenticated:
            return []

        groups = request.user.groups.all().values_list('name', flat=True)
        return list(groups)

    def location_data_from_ip_address(self, ip_address: str) -> dict:
        location_data: dict = {}

        if not ip_address or not CAN_GEOIP:
            return location_data

        # pip install geoip2
        # https://pypi.org/project/geoip2/
        # from django.contrib.gis.geoip2 import GeoIP2
        g = GeoIP2()
        location = g.city(ip_address)
        location_data['country'] = location['country_name']
        location_data['city'] = location['city']
        location_data['region'] = g.region(ip_address)
        lat_lon = g.lat_lon(ip_address)
        location_data['location_lat'] = lat_lon[0]
        location_data['location_lng'] = lat_lon[1]
        return location_data

    def device_data_from_request(self, request) -> dict:
        device_data: dict = {}

        user_agent = get_user_agent(request)
        if not user_agent:
            return device_data

        device_data['os_name'] = user_agent.os.family
        device_data['os_version'] = user_agent.os.version_string
        device_data['platform'] = user_agent.device.family
        # device_data['device_brand']
        device_data['device_manufacturer'] = user_agent.device.brand
        device_data['device_model'] = user_agent.device.model
        return device_data



from . import Amplitude

amplitude = Amplitude()


class SendPageViewEvent(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        amplitude.page_view_event(request)
        return self.get_response(request)



from typing import Dict

try:
    from user_agents import parse as user_agent_parse  # type: ignore
except ImportError:  # pragma: no cover
    USER_AGENT_AVAILABLE = False
else:
    KNOWN_USER_AGENTS: Dict[str, str] = {}
    USER_AGENT_AVAILABLE = True


def get_client_ip(request) -> str:
    if not hasattr(request, 'META'):
        return ''

    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[-1].strip()
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def get_user_agent(request):
    if not hasattr(request, 'META'):
        return ''

    http_user_agent = request.META.get('HTTP_USER_AGENT', '')
    if http_user_agent:
        return ''

    if http_user_agent in KNOWN_USER_AGENTS:
        return KNOWN_USER_AGENTS[http_user_agent]

    if not USER_AGENT_AVAILABLE:
        return ''

    user_agent = user_agent_parse(http_user_agent)
    KNOWN_USER_AGENTS[http_user_agent] = user_agent

    return user_agent



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['amplitude']

package_data = \
{'': ['*']}

install_requires = \
['Django>=2.1', 'httpx>=0.13.2,<0.14.0', 'user-agents>=2.1,<3.0']

setup_kwargs = {
    'name': 'django-amplitude',
    'version': '0.2.0',
    'description': 'Integration between Django and Amplitude',
    'long_description': '# Django Amplitude\n\nIntegration between Django and [Amplitude.com](https://amplitude.com/) to help send events via the [Amplitude HTTP API (v2)](https://developers.amplitude.com/docs/http-api-v2)\n\n\n## Quick start\n\n### Installation\n\n```bash\npip install django-amplitude\n```\n\nSet a Amplitude API key in your Django settings:\n```python\n# Settings > Projects > <Your project> > General > API Key\nAMPLITUDE_API_KEY = \'<amplitude-project-api-key>\'\n\n# You can also choose if you want to include user and group data\n# IF not set will default to False\nAMPLITUDE_INCLUDE_USER_DATA = True\nAMPLITUDE_INCLUDE_GROUP_DATA = True\n```\n\nAdd `amplitude` to your `INSTALLED_APPS`:\n\n```python\nINSTALLED_APPS =(\n    ...\n    "amplitude",\n    ...\n)\n```\n\n\n## Usage\n\nIf you want to send an event to Amplitude on every page view you can use the django-amplitude `SendPageViewEvent` middleware to your `MIDDLEWARE` in your Django settings.\nThis will automatically create an event base on the url name that was hit and the Django request object.\n\n```python\nMIDDLEWARE = [\n    ...\n    \'amplitude.middleware.SendPageViewEvent\',\n    ...\n]\n```\n\nIf you want to send your own events:\n```python\nfrom amplitude import Amplitude\n\namplitude = Amplitude()\namplitude.send_events([event_data])  # https://developers.amplitude.com/docs/http-api-v2\n```\n\nThere are also few helper functions to build different parts of the event data:\n```python\namplitude.session_id_from_request(request)\namplitude.event_properties_from_request(request)\namplitude.device_data_from_request(request)\namplitude.user_properties_from_request(request)\namplitude.group_from_request(request)\n\namplitude.location_data_from_ip_address(ip_address)\n```\n\n* `user_properties_from_request` will return an empty dict is `AMPLITUDE_INCLUDE_USER_DATA` is `False`\n* `group_from_request` will return an empty dict is `AMPLITUDE_INCLUDE_GROUP_DATA` is `False`\n\n\n\n### SendPageViewEvent - missing event data keys\n\nThe `SendPageViewEvent` middleware currently does not record the following keys from `UploadRequestBody`:\n\n* event_id\n* app_version\n* device_id\n* carrier\n* price\n* quantity\n* revenue\n* productId\n* revenueType\n* idfa\n* idfv\n* adid\n* android_id\n* dma\n* insert_id\n\nIf you want to record an event in Amplitude with any of these keys you must use `amplitude.send_events([event_data])`\n',
    'author': 'Matt Pye',
    'author_email': 'pyematt@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pyepye/django-amplitude',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
